package Bro::Log;

require 5.006_001;
use strict;
use Bro::Config( '$BRO_CONFIG' );

use vars qw( $VERSION
			$BROLOGS );

# $Id: Log.pm,v 1.6 2004/10/13 06:00:57 rwinslow Exp $
$VERSION = 1.05;



my $FILENAME_REGEX = qr/^([^.]+)\.	# log type
		(.+)\.				# hostname, and other stuff
		([[:digit:]]+)-([[:digit:]]+)-([[:digit:]]+)	# year, month, day
		_	# seperator
		([[:digit:]]+)\.([[:digit:]]+)\.([[:digit:]]+)	# hour, minute, second
		(?:		# There may also be an ending time
		\-		# seperator
		([[:digit:]]+)-([[:digit:]]+)-([[:digit:]]+)	# year, month, day
		_	# seperator
		([[:digit:]]+)\.([[:digit:]]+)\.([[:digit:]]+)	# hour, minute, second
		)?	# End of optional ending time
		([^[:space:][:digit:]\.]+)?$/xo;	# other

sub activelog
{
	my $sub_name = 'activelog';

	my $log_dir = $BRO_CONFIG->{BROLOGS};
	my $ret_str;

	if( !( defined( $log_dir ) ) )
	{
		warn( "no log directory defined\n" );
		return( undef );
	}

	if( -f "$log_dir/active_log" )
	{
		if( open( I_FILE, "$log_dir/active_log" ) )
		{
			if( defined( $ret_str = <I_FILE> ) )
			{
				# remove any trailing newlines
				if( $ret_str !~ m/^[[:space]]*$/ )
				{
					chomp( $ret_str );
				}
				else
				{
					return( 0 );
				}
			}
			else
			{
				return( 0 );
			}
		}
		else
		{
			warn( "Failed to read the active log file at $log_dir/active_log\n" );
		}

		close( I_FILE );
	}
	else
	{
		return( 0 );
	}

	return( $ret_str );
}

sub loglist
{
	my $sub_name = 'log_list';

	my $__log_type = $_[0] || return( undef );
	my $brologs_dir = $BRO_CONFIG->{BROLOGS};
	my @ret_list;

	if( opendir( DIR, $brologs_dir ) )
	{
		while( defined( my $file_name = readdir( DIR ) ) )
		{
			if( $file_name =~ $FILENAME_REGEX )
			{
				my $log_type = $1;
				if( $log_type eq $__log_type )
				{
					push( @ret_list, "$brologs_dir/$file_name" );
				}
			}
		}
	}
	else
	{
		warn( __PACKAGE__ . "::$sub_name, Unable to open the BROLOGS directory\n" );
		return( undef );
	}

	closedir( DIR );

	if( wantarray )
	{
		return( @ret_list );
	}
	else
	{
		return( \@ret_list );
	}
}

sub filetimestamptoepoch
{
	my $sub_name = 'filetimestamptoepoch';

	my $filename = $_[0] || return( undef );
	my $epoch_time;

	my( $log_type, $hostname, $year, $mon, $day, $hour, $min, $sec, $other ) = 
		( $filename =~ $FILENAME_REGEX );

	# If the file has been rotated by Bro then it will have the actual timestamp
	# in the "other" element
	if( length( $other ) > 18 )
	{
		my( $pid, $time, $junk ) = split( /\./, $other, 3 );
		$epoch_time = $time;

	}
	elsif( $mon and $day )
	{
		# The month fed into timelocal is 0 based index
		if( $mon > 0 )
		{
			--$mon;
		}
		$epoch_time = timelocal($sec,$min,$hour,$day,$mon,$year);
	}

	return( $epoch_time );
}

1;
