package Bro::Log::Alarm;

use strict;
require 5.006_001;
use strict;

use vars qw( $VERSION
		%DATA_MAP );

# $Id: Alarm.pm,v 1.8 2004/10/12 17:10:20 rwinslow Exp $
$VERSION = 1.05;

# Map data descriptions to subroutine names
%DATA_MAP = ( t => \&timestamp,
			timestamp => \&timestamp,
			alert => \&alert_type,
			alert_type => \&alert_type,
			alert_act => \&alert_action,
			alert_action => \&alert_action,
			event_src => \&event_source,
			event_source => \&event_source,
			src_addr => \&source_ip,
			srcip => \&source_ip,
			source_ip => \&source_ip,
			src_port => \&source_port,
			source_port => \&source_port,
			dst_addr => \&destination_ip,
			dstip => \&destination_ip,
			destination_ip => \&destination_ip,
			dst_port => \&destination_port,
			destination_port => \&destination_port,
			user => \&user,
			filename => \&filename,
			sigid => \&sigid,
			method => \&method,
			URL => \&url,
			n => \&misc_integer,
			count => \&misc_integer,
			return_code => \&misc_integer,
			msg => \&message,
			message => \&message,
			sub_msg => \&sub_message,
			sub_message => \&sub_message,
			);

sub new
{
	my $_log_line;
	my @_args = @_;
	my @log_parts;

	if( @_args == 1 )
	{
		$_log_line = $_args[0];
	}
	else
	{
		return( undef );
	}

	# Order of data in array
	# 0 = t, timestamp
	# 1 = alert, alert_type
	# 2 = alert_act, alert_action
	# 3 = event_src, event_source
	# 4 = src_addr, source_ip
	# 5 = src_port, source_port
	# 6 = dst_addr, destination_ip
	# 7 = dst_port, destination_port
	# 8 = user, user
	# 9 = filename, filename or sigid
	# 10 = method, method
	# 11 = URL, URL
	# 12 = n, count or return_code
	# 13 = msg, message
	# 14 = sub, sub_message

	if( $_log_line =~ m/\\:/ )
	{
		my $i = 0;
		my $i2 = 0;
		my $len = length( $_log_line );
		my $p_idx = 0;
		my $buff_pos = 0;
		my $subtr_len = 0;
		for( $i2 = 0; $i2 < $len; ++$i2 )
		{
			if( substr( $_log_line, $i2, 1 ) eq ':' and
				substr( $_log_line, $p_idx, 1 ) ne "\\" )
			{
				if( $subtr_len < 1 )
				{
					$log_parts[$i] = '';
				}
				else
				{
					$log_parts[$i] = substr( $_log_line, $buff_pos, $subtr_len );
					$log_parts[$i] = unescape( $log_parts[$i] );
				}
				$subtr_len = 0;
				$p_idx = $i2 + 1;
				$buff_pos = $i2 + 1;
				++$i;
			}
			else
			{
				++$subtr_len;
				$p_idx = $i2;				
			}
		}

		# Get the last piece of data
		$log_parts[$i] = substr( $_log_line, $buff_pos, $subtr_len );
		$log_parts[$i] =~ s/\\:/:/g;
	}
	else
	{
		@log_parts = split( /:/, $_log_line, 15 );
	}
	
	if( defined( $log_parts[2] ) )
	{
		return( \@log_parts );
	}
	else
	{
		return( undef );
	}
}

sub output
{
	my $sub_name = 'output';

	my $data = $_[0] || return undef;
	my $format = $_[1];
	my @ret_data;

	if( ref( $format ) ne 'ARRAY' )
	{
		$format = [ 'timestamp',
				'alert_type',
				'alert_action',
				'event_source',
				'source_ip',
				'source_port',
				'destination_ip',
				'destination_port',
				'user',
				'filename',
				'method',
				'url',
				'misc_integer',
				'message',
				'sub_message',
				];

		@ret_data = @{$data};
	}
	else
	{
		foreach my $key( @{$format} )
		{
			if( exists( $DATA_MAP{$key} ) )
			{
				push( @ret_data, &{$DATA_MAP{$key}}( $data ) );
			}
			else
			{
				return( undef );
			}
		}
	}

	if( wantarray )
	{
		return( @ret_data );
	}
	else
	{
		return( join( ' ', @ret_data ) );
	}
}

sub timestamp
{
	my $sub_name = 'timestamp';

	my $data = $_[0];
	my $format = $_[1];

	return( $data->[0] );
}

sub alert_type
{
	my $sub_name = 'alert_type';

	my $data = $_[0] || return( undef );

	return( $data->[1] );
}

sub alert_action
{
	my $sub_name = 'alert_action';

	my $data = $_[0] || return( undef );

	return( $data->[2] );
}

sub event_source
{
	my $sub_name = 'event_source';

	my $data = $_[0] || return( undef );

	return( $data->[3] );
}

sub source_ip
{
	my $sub_name = 'source_ip';

	my $data = $_[0] || return( undef );

	return( $data->[4] );
}

sub source_port
{
	my $sub_name = 'source_port';

	my $data = $_[0] || return undef;

	return( $data->[5] );
}

sub destination_ip
{
	my $sub_name = 'destination_ip';

	my $data = $_[0] || return undef;

	return( $data->[6] );
}

sub destination_port
{
	my $sub_name = 'destination_port';

	my $data = $_[0] || return undef;

	return( $data->[7] );
}

sub user
{
	my $sub_name = 'user';

	my $data = $_[0] || return undef;

	return( $data->[8] );
}

sub filename
{
	my $sub_name = 'filename';

	my $data = $_[0] || return undef;

	return( $data->[9] );
}

sub sigid
{
	my $sub_name = 'sigid';
	
	&filename;
}

sub method
{
	my $sub_name = 'method';

	my $data = $_[0] || return undef;

	return( $data->[10] );
}

sub url
{
	my $sub_name = 'url';

	my $data = $_[0] || return undef;

	return( $data->[11] );
}

sub misc_integer
{
	my $sub_name = 'misc_integer';

	my $data = $_[0] || return undef;

	return( $data->[12] );
}

sub count
{
	&misc_integer;
}

sub return_code
{
	&misc_integer;
}

sub message
{
	my $sub_name = 'message';

	my $data = $_[0] || return undef;

	return( $data->[13] );
}

sub sub_message
{
	my $sub_name = 'sub_message';

	my $data = $_[0] || return undef;
	chomp( $data->[14] );

	return( $data->[14] );
}

sub unescape
{
	my $sub_name = 'unescape';
	
	my $data = $_[0];
	
	if( ! defined( $data ) )
	{
		return( undef );
	}
	else
	{
		$data =~ s/\\:/:/g;
		$data =~ s/\\\\/\\/g;
	}
	
	return( $data );
}

sub timerange
{
	my $sub_name = 'timerange';
	# Find the most likely beginning and ending times covered by a given
	# conn file.

	my $filename = $_[0];
	my $start_time = 9999999999;
	my $end_time = -1;
	my $f_size = ( stat( $filename ) )[7];
	
	if( open( INFILE, $filename ) )
	{
		my $s_idx = 0;
		my $s_no_change = 0;

		# Find the smallest timestamp in the first 1000 lines.
		while( defined( my $ln = <INFILE> ) and
			( $s_idx < 1000 ) and
			( $s_no_change < 20 ) )
		{
			if( my $alarm_line = new( $ln ) )
			{
				my $w_timestamp = timestamp( $alarm_line );
				if( $w_timestamp < $start_time )
				{
					$start_time = $w_timestamp;
					$s_no_change = 0;
				}
				else
				{
					++$s_no_change;
				}
			}

			++$s_idx;
		}

		close( INFILE );

		# Find the largest timestamp in the last 1000 lines
		# Each connection with a status of "SF" will be counted as one line
		# Every line will be examined but the "SF" lines are the only ones
		# that give a good picture as to the time state of the file.
		if( sysopen( INFILE, $filename, 0 ) )
		{
			sysseek( INFILE, $f_size, 0 );
			my $cur_pos = sysseek( INFILE, 0, 1 );
			my $nl_pos = $cur_pos;
			my $line_count = 0;
			my $e_no_change = 0;

			# Get last 1000 lines
			while( $line_count < 1000 and $e_no_change < 20 )
			{
				my $new_line_found = 0;
				my $buf;
				sysread( INFILE, $buf, 1 );

				if( $cur_pos > -1 )
				{
					if( $buf eq $/ )
					{
						$new_line_found = 1;
					}
				}
				else
				{
					# Must have hit the beginning of the file
					if( $nl_pos > 20 )
					{
						$cur_pos = 0;
						sysseek( INFILE, 0, 0 );
						$new_line_found = 1;
					}
					else
					{
						last;
					}
				}

				if( $new_line_found )
				{
					my $cur_line = '';
					sysread( INFILE, $cur_line, $nl_pos - $cur_pos );
					if( my $alarm_line = new( $cur_line ) )
					{
						my $w_timestamp = timestamp( $alarm_line );
						if( $w_timestamp > $end_time )
						{
							$end_time = $w_timestamp;
						}
						else
						{
							++$e_no_change;
						}
					}
					$nl_pos = $cur_pos;
					++$line_count;
				}
				--$cur_pos;
				if( $cur_pos < 0 )
				{
					last;
				}
				sysseek( INFILE, $cur_pos, 0 );
			}

		}
		else
		{
			warn( __PACKAGE__ . "::$sub_name, Unable to open file '$filename' with sysread.\n" );
			return( undef );
		}

		close( INFILE );
	}
	else
	{
		warn( __PACKAGE__ . "::$sub_name, Unable to open file '$filename'.\n" );
		return( undef );
	}
	
	# Make sure that sane values were found for the start and end times
	if( $start_time == 9999999999 or $end_time == -1 )
	{
		# warn( __PACKAGE__ . "::$sub_name, There was an error determining the start and end ranges.\n" );
		# warn( "No valid values could be found.\n" );
		return( undef );
	}
	
	return( $start_time, $end_time );
}

1;
